//----------------------------------------------------------------------------
// WinMDCD.cpp
// (C) 2006 Mark Horka
// http://www.antistatic.org/
//
// WinMDCD is a MDC-1200 software-based decoder.
// This software was written using the DirectX SDK
//
//-----------------------------------------------------------------------------

/*
 *  The Windows MDC-1200 Decoder is free software; you can
 *  redistribute it and/or modify it under the terms of version 2 of
 *  the GNU General Public License as published by the Free Software
 *  Foundation.
 *
 *  If you cannot comply with the terms of this license, contact
 *  the author for alternative license arrangements or do not use
 *  or redistribute this software.
 *
 *  The Windows MDC-1200 Decoder is distributed in the hope
 *  that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE.  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA.
 *
 *  or see http://www.gnu.org/copyleft/gpl.html
 */

#include "dxstdafx.h"
#include <commdlg.h>
#include "resource.h"
#include "mdc_decode.h"

// user defines
#define IDC_LIST1			WM_USER + 1
#define WM_USER_SHELLICON	WM_USER + 2

//-----------------------------------------------------------------------------
// Function-prototypes
//-----------------------------------------------------------------------------
HRESULT		InitDirectSound( HWND hDlg, GUID* pDeviceGuid );
HRESULT		FreeDirectSound();
INT_PTR		CALLBACK MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT		OnInitMainDialog( HWND hDlg );
BOOL		NewLogFile( HWND hDlg);
HRESULT		CreateCaptureBuffer( WAVEFORMATEX* pwfxInput );
HRESULT		InitNotifications();
HRESULT		StartOrStopRecord( BOOL bStartRecording );
HRESULT		RecordCapturedData();
VOID		SetupListBox(HWND hDlg );
HWND		CreateListView(HWND hDlg);
VOID		AddEvent( HWND hwndList, unsigned int eid, unsigned char eop, unsigned char earg);
VOID		SetRegistryParameters();

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
#define NUM_REC_NOTIFICATIONS  16
#define MAX(a,b)        ( (a) > (b) ? (a) : (b) )

/* little help with the op/arg determination.. */
#define SARG(s)			(lstrcmpi(sarg, TEXT(#s)) == 0)
#define SOP(s)			(lstrcmpi(sop, TEXT(#s)) == 0)
#define SOPARG(o,a)		((lstrcmpi(sop, TEXT(#o)) == 0) && (lstrcmp(sarg, TEXT(#a)) == 0))
#define DESC(d)			StringCchPrintf(sdesc, strlen(#d)+1, TEXT(#d));

#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p)	{ if(p) { (p)->Release(); (p)=NULL; } }

LPDIRECTSOUNDCAPTURE		g_pDSCapture         = NULL;
LPDIRECTSOUNDCAPTUREBUFFER	g_pDSBCapture        = NULL;
LPDIRECTSOUNDNOTIFY		g_pDSNotify          = NULL;
HINSTANCE				g_hInst              = NULL;
GUID					g_guidCaptureDevice  = GUID_NULL;
WAVEFORMATEX			g_wfxInput;
DSBPOSITIONNOTIFY		g_aPosNotify[ NUM_REC_NOTIFICATIONS + 1 ];
HANDLE					g_hNotificationEvent;
DWORD					g_dwCaptureBufferSize;
DWORD					g_dwNextCaptureOffset;
DWORD					g_dwNotifySize;
BOOL					g_bScroll = true;
BOOL					g_bLog = false;
NOTIFYICONDATA			g_id;
TCHAR					g_LogFileName[MAX_PATH];
HKEY					g_hKey;

/* for the MDC decoder library */
mdc_decoder_t			*decoder = mdc_decoder_new(22050);
int						result, result2;
unsigned char			op, arg;
unsigned short			unitID;
char					uID[5];

//-----------------------------------------------------------------------------
// Name: WinMain()
//-----------------------------------------------------------------------------
INT APIENTRY WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR pCmdLine,
                      INT nCmdShow )
{
    HRESULT hr;
    DWORD dwResult;
    MSG   msg;
    BOOL  bDone;
    HWND  hDlg;

	g_hInst = hInst;

    g_hNotificationEvent = CreateEvent( NULL, FALSE, FALSE, NULL );

	InitCommonControls();

    // Create the main dialog box, but keep it hidden for now
    hDlg = CreateDialog( hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)MainDlgProc );	

        // Init DirectSound
		GUID* pCaptureGUID = (GUID*) 0;
		// Remember that guid
		if( pCaptureGUID )
			g_guidCaptureDevice = *pCaptureGUID;
        if( FAILED( hr = InitDirectSound( hDlg, &g_guidCaptureDevice ) ) )
        {
            DXTRACE_ERR_MSGBOX( TEXT("InitDirectSound"), hr );
            MessageBox( hDlg, L"Error initializing DirectSound.  Sample will now exit.",
                        L"DirectSound Sample", MB_OK | MB_ICONERROR );
            PostQuitMessage( 0 );
            dwResult = IDCANCEL;
        }

	// set wave format
	g_wfxInput.wFormatTag = WAVE_FORMAT_PCM;
	g_wfxInput.wBitsPerSample = 8;
	g_wfxInput.nSamplesPerSec = 22050;
	g_wfxInput.nChannels = 1;
	g_wfxInput.nBlockAlign = 1;
	g_wfxInput.nAvgBytesPerSec = 22050;


	// init icon
    HICON hIconSmall = LoadIcon( g_hInst, MAKEINTRESOURCE( IDI_ICONSMALL ) );
	ZeroMemory(&g_id,sizeof(NOTIFYICONDATA));
	g_id.cbSize = sizeof(NOTIFYICONDATA);
	g_id.hWnd = hDlg;
	g_id.hIcon = hIconSmall;
	g_id.uID = IDI_ICONSMALL;
	g_id.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
	g_id.uCallbackMessage = WM_USER_SHELLICON;
	TCHAR iconText[8] = TEXT("WinMDCD");
	StringCchCopy(g_id.szTip, sizeof(iconText), iconText);	

	// create list
	HWND hwndList = CreateListView(hDlg);

	// initialize 
	StartOrStopRecord(TRUE);
	ShowWindow( hDlg, SW_SHOW );
    bDone = FALSE;
    while( !bDone )
    {
        dwResult = MsgWaitForMultipleObjects( 1, &g_hNotificationEvent,
                                            FALSE, INFINITE, QS_ALLEVENTS );

                if( FAILED( hr = RecordCapturedData() ) )
                {
                    DXTRACE_ERR_MSGBOX( TEXT("RecordCapturedData"), hr );
                    MessageBox( hDlg, L"Error handling DirectSound notifications. "
                               L"Sample will now exit.", L"DirectSound Sample",
                               MB_OK | MB_ICONERROR );
                    bDone = TRUE;
                }

				// TODO: add support for buffer 2
				// TODO: add double packet support
				if (result == 1)
				{
					// if an MDC was detected, show it
					AddEvent(hwndList, unitID, op, arg);
					result = result2 = 0;
				}

				while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
				{

                    if( !IsDialogMessage( hDlg, &msg ) )
                    {
                        TranslateMessage( &msg );
                        DispatchMessage( &msg );
                    }

                    if( msg.message == WM_QUIT )
					{
                        bDone = TRUE;
						PostQuitMessage(0);
						break;
					}
                }
    }

    // Stop the capture and read any data that was not caught by a notification
    StartOrStopRecord( FALSE );

    // Clean up everything
    FreeDirectSound();

	// Save settings to registry
	SetRegistryParameters();

	// remove the icon if there is one
	Shell_NotifyIcon(NIM_DELETE, &g_id);

	// stop message pump
    CloseHandle( g_hNotificationEvent );

	// quit cleanly
    return TRUE;
}

// SetRegistryParameters()
// - Saves the filename, and status of log/scroll checkboxes to:
// HLCU\AntiStatic\WinMDCD
VOID SetRegistryParameters()
{
	unsigned int pathLength = MAX_PATH;
	RegSetValueEx(g_hKey, TEXT("FileName"), NULL, REG_SZ, (BYTE*)g_LogFileName, pathLength);
	RegSetValueEx(g_hKey, TEXT("Log"), NULL, REG_DWORD, (CONST BYTE*)&g_bLog, sizeof(DWORD));
	RegSetValueEx(g_hKey, TEXT("Scroll"), NULL, REG_DWORD, (CONST BYTE*)&g_bScroll, sizeof(DWORD));
}


//-----------------------------------------------------------------------------
// Name: InitDirectSound()
// Desc: Initilizes DirectSound
//-----------------------------------------------------------------------------
HRESULT InitDirectSound( HWND hDlg, GUID* pDeviceGuid )
{
    HRESULT hr;

    ZeroMemory( &g_aPosNotify, sizeof(DSBPOSITIONNOTIFY) *
                               (NUM_REC_NOTIFICATIONS + 1) );
    g_dwCaptureBufferSize = 0;
    g_dwNotifySize        = 0;

    // Initialize COM
    if( FAILED( hr = CoInitialize(NULL) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("CoInitialize"), hr );


    // Create IDirectSoundCapture using the preferred capture device
    if( FAILED( hr = DirectSoundCaptureCreate( pDeviceGuid, &g_pDSCapture, NULL ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("DirectSoundCaptureCreate"), hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FreeDirectSound()
// Desc: Releases DirectSound
//-----------------------------------------------------------------------------
HRESULT FreeDirectSound()
{
    // Release DirectSound interfaces
    SAFE_RELEASE( g_pDSNotify );
    SAFE_RELEASE( g_pDSBCapture );
    SAFE_RELEASE( g_pDSCapture );

    // Release COM
    CoUninitialize();

    return S_OK;
}

VOID AddEvent( HWND hwndList, unsigned int eid, unsigned char eop, unsigned char earg)
{
	// this is just a mess..
    LV_ITEM item;
    LONG iSubItem;
	TCHAR sid[5];
	TCHAR sop[3];
	TCHAR sarg[3];
	TCHAR sdesc[64];
	LPCTSTR pszFormat = TEXT("%02X");
	StringCchPrintf(sid, 5, pszFormat, eid);
	StringCchPrintf(sop, 3, pszFormat, eop);
	StringCchPrintf(sarg, 3, pszFormat, earg);
	SYSTEMTIME st;
	GetLocalTime(&st);
	TCHAR pszDate[20];
	LPCTSTR pszDateFormat = TEXT("%02d/%02d/%04d %02d:%02d:%02d");
	StringCchPrintf(pszDate, 20, pszDateFormat,
		st.wMonth, st.wDay, st.wYear, st.wHour, st.wMinute, st.wSecond);

    iSubItem = 0;
    item.mask = LVIF_TEXT | LVIF_STATE | LVIS_FOCUSED;
    item.iItem = ListView_GetItemCount(hwndList);
    item.stateMask = 0xffff;
    item.state = 0;

    item.iSubItem = iSubItem++;
    item.pszText = pszDate;
    if (-1 == ListView_InsertItem(hwndList, &item))
        return;

    item.iSubItem = iSubItem++;
    item.pszText = sop;
    if (-1 == ListView_SetItem(hwndList, &item))
        return;

    item.iSubItem = iSubItem++;
    item.pszText = sarg;
    if (-1 == ListView_SetItem(hwndList, &item))
        return;

    item.iSubItem = iSubItem++;
    item.pszText = sid;
    if (-1 == ListView_SetItem(hwndList, &item))
        return;

    item.iSubItem = iSubItem++;

	// default description string to nothing.
	StringCchPrintf(sdesc, 1, TEXT(""));

	// the following is a ruler.. it goes to            Y	
  //if (SOPARG(01,80)) { DESC(ABCDEFGHIJKLMNOPQRSTUVWXYZ01234567890) }

	if (SOPARG(01,80)) { DESC(PRE UNIT ID) }
	if (SOPARG(01,00)) { DESC(POST UNIT ID) }

	if (SOPARG(00,80)) { DESC(EMERGENCY ALARM) }
	if (SOPARG(00,81)) { DESC(EMERGENCY ALARM [XTS3000]) }

	if (SOPARG(20,00)) { DESC(EMERGENCY ALARM ACKNOWLEDGE) }
	
	if (SOPARG(63,85)) { DESC(RADIO CHECK) }

	if (SOPARG(03,00)) { DESC(RADIO CHECK ACKNOWLEDGE) }

	// NOT FULLY IMPLEMENTED
	// TODO: implement double packets
	if (SOPARG(35,89)) { DESC(CALL - not implemented) }

	//if (SOPARG(46,0X)) { DESC(SIMPLE STATUS) }
	
	if (SOPARG(23,00)) { DESC(MESSAGE ACKNOWLEDGE) }

	if (SOPARG(22,06)) { DESC(STATUS REQUEST) }

	//if (SOP(06) && SARG(0)) { DESC(STATUS RESPONSE) }
	
	//if (SOPARG(47,0X)) { DESC(INBOUND MESSAGE ACK EXPECTED) }
	//if (SOPARG(07,0X)) { DESC(INBOUND MESSAGE ACK NOT EXPECTED) }

	if (SOPARG(11,8A)) { DESC(REMOTE MONITOR) }

	if (SOPARG(2B,00)) { DESC(INHIBIT) }
	if (SOPARG(0B,00)) { DESC(INHIBIT ACKNOWLEDGE) }
	if (SOPARG(2B,0C)) { DESC(INHIBIT CANCEL) }
	if (SOPARG(0B,0C)) { DESC(INHIBIT CANCEL ACKNOWLEDGE) }

	if (SOPARG(40,01)) { DESC(REQUEST TO TALK - BUTTON) }
	if (SOPARG(41,01)) { DESC(REQUEST TO TALK - MIC PTT) }
	
	// DUPLICATE
	//if (SOPARG(23,00)) { DESC(REQUEST TO TALK ACKNOWLEDGE) }

	// show description
    item.pszText = sdesc;
    if (-1 == ListView_SetItem(hwndList, &item))
        return;


	// If scrolling is turned on, scroll down
	if (g_bScroll)
		ListView_EnsureVisible(hwndList, item.iItem, TRUE);

	// If logging is turned on, log it
	// NOTE: if the file does not exist, it will be created
	if (g_bLog)
	{
	HANDLE hFile = CreateFile(
		(LPCTSTR)g_LogFileName,
		GENERIC_WRITE,
		FILE_SHARE_READ,
		NULL,
        OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL);

	DWORD dwWritten;
	
	// dont even ask about this next part....
	TCHAR strText[98];
	if (hFile != INVALID_HANDLE_VALUE)
	{
		size_t strSize;
		SetFilePointer(hFile, 0, NULL, FILE_END);
		pszFormat = TEXT("%02d/%02d/%04d\t%02d:%02d:%02d\t%02X\t%02X\t%04X\t%s\r\n");
		StringCchPrintf(strText, 98, pszFormat,
			st.wMonth, st.wDay, st.wYear, st.wHour, st.wMinute, st.wSecond,
			eop, earg, eid, sdesc);
		StringCchLength(strText, STRSAFE_MAX_CCH, &strSize);
		WriteFile(hFile, (LPCTSTR)strText, strSize*2, &dwWritten, NULL);
	}
	CloseHandle(hFile);
	}
}

//-----------------------------------------------------------------------------
// Name: MainDlgProc()
// Desc: Handles dialog messages
//-----------------------------------------------------------------------------
INT_PTR CALLBACK AboutDlgProc
   (HWND hwnd, UINT	message, UINT wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD (wParam))
		{
		case IDOK:
		case IDCANCEL:
			::EndDialog	(hwnd, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

INT_PTR CALLBACK MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
    //HRESULT hr;

    switch( msg )
    {
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

	case WM_CLOSE:
		PostQuitMessage(0);
		DestroyWindow(hDlg);
		break;

	case WM_USER_SHELLICON:
	
		switch(LOWORD(lParam))
		{
		case WM_LBUTTONDBLCLK: // to open the GUI of the Application
            ShowWindow(hDlg, SW_SHOW);
			Shell_NotifyIcon(NIM_DELETE, &g_id);
			return TRUE;
		}

	return FALSE;

	case WM_SYSCOMMAND:
		wParam &= 0xfff0; 
		switch (wParam)
		{
		case SC_MINIMIZE:
			Shell_NotifyIcon(NIM_ADD, &g_id);
			ShowWindow(hDlg,SW_HIDE);
			return TRUE;
		
		case SC_CLOSE:
            PostQuitMessage( 0 );
            EndDialog(hDlg, IDCANCEL );
			return TRUE;
		}
		return FALSE;
    case WM_INITDIALOG:
        OnInitMainDialog( hDlg );
        break;

    case WM_COMMAND:
        switch( LOWORD(wParam) )
        {
            case IDCANCEL:
                PostQuitMessage( 0 );
                EndDialog(hDlg, IDCANCEL );
                break;
			case IDC_FILENAME:
				system("notepad.exe");
				break;
			case IDC_CLEAR:
				// note: clear does not delete items from the log
				ListView_DeleteAllItems(GetDlgItem(hDlg, IDC_LIST1));
				break;
			case IDC_ABOUT:
			    DialogBox(g_hInst, MAKEINTRESOURCE(IDD_ABOUT), NULL, AboutDlgProc );
				break;
			case IDC_NEWFILE:
				if (NewLogFile(hDlg))
				SetDlgItemText(hDlg, IDC_FILENAME, g_LogFileName);
				break;
			case IDC_LOG:
				if (IsDlgButtonChecked(hDlg, IDC_LOG) == BST_CHECKED)
				{
					if (lstrcmp(g_LogFileName, TEXT("")) == 0)
					{
						if (!NewLogFile(hDlg))
							break;
					}
					SetDlgItemText(hDlg, IDC_LOGTO, TEXT("Logging to:"));
					SetDlgItemText(hDlg, IDC_FILENAME, g_LogFileName);
					g_bLog = true;
				}
				else
				{
					SetDlgItemText(hDlg, IDC_LOGTO, TEXT("Not logging."));
					SetDlgItemText(hDlg, IDC_FILENAME, TEXT(""));
					g_bLog = false;

				}
				break;

			case IDC_SCROLL:
				if (IsDlgButtonChecked(hDlg, IDC_SCROLL) == BST_CHECKED)
					g_bScroll = true;
				else g_bScroll = false;
				break;
            default:
                return FALSE; // Didn't handle message
        }
        break;

    default:
        return FALSE; // Didn't handle message
    }
    return TRUE; // Handled message
}


BOOL NewLogFile(HWND hDlg)
{
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hDlg;
	ofn.lpstrFilter = TEXT("Log files (*.txt,*.log)\0*.txt;*.log\0All Files (*.*)\0*.*\0");
	ofn.lpstrFile = g_LogFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = TEXT("txt");
	if(!GetSaveFileName(&ofn))
	{
		MessageBox(hDlg, TEXT("Error opening file."), TEXT("Error"), MB_OK | MB_ICONERROR);
		return false;
	}

	HANDLE hFile = CreateFile((LPCTSTR) g_LogFileName,  // filename
	GENERIC_WRITE, // open for write
	0,                            // do not share
	NULL,                         // no security
	CREATE_ALWAYS,                // overwrite existing file
	FILE_ATTRIBUTE_NORMAL,        // normal file
	NULL);                        // no attr. template
	CloseHandle(hFile);
	g_bLog = true;
	return true;
}

//-----------------------------------------------------------------------------
// Name: OnInitMainDialog()
// Desc: Initializes the main dialog
//-----------------------------------------------------------------------------
HRESULT OnInitMainDialog( HWND hDlg )
{
	DWORD lValueFile = MAX_PATH;
	DWORD lValueLog = sizeof(DWORD), lValueScroll = sizeof(DWORD);
	TCHAR lpFileData[MAX_PATH];
	DWORD lpLogData, lpScrollData;
	DWORD datatype, datatype2, datatype3;
	unsigned long dwDisp;

	// Load the icon
    HICON hIconSmall = LoadIcon( g_hInst, MAKEINTRESOURCE( IDI_ICONSMALL ) );
	HICON hIconBig = LoadIcon( g_hInst, MAKEINTRESOURCE( IDI_ICONBIG ) );
    // Set the icon for this dialog.
	SendMessage( hDlg, WM_SETICON, ICON_SMALL, (LPARAM) hIconSmall );  // Set small icon
	SendMessage( hDlg, WM_SETICON, ICON_BIG,   (LPARAM) hIconBig );  // Set big icon

	// Load the  registry value, if its not there create it
	// if we cant create it, RUN AWAY!!!!!
	if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("SOFTWARE\\AntiStatic\\WinMDCD"),NULL,KEY_ALL_ACCESS, &g_hKey)!=ERROR_SUCCESS)
	{
		if  (RegCreateKeyEx(HKEY_CURRENT_USER, TEXT("SOFTWARE\\AntiStatic\\WinMDCD"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
               NULL, &g_hKey, &dwDisp) != ERROR_SUCCESS)
		{
			MessageBox(hDlg, TEXT("FATAL REGISTRY ERROR"), TEXT("FATAL REGISTRY ERROR"), MB_OK);
		}
	}

	// Set scrolling default to scrolling
	SendDlgItemMessage( hDlg, IDC_SCROLL, BM_SETCHECK, BST_CHECKED, 0);
	if (RegQueryValueEx(g_hKey, TEXT("Scroll"),  NULL, &datatype3, (BYTE*)&lpScrollData, &lValueScroll)==ERROR_SUCCESS)
	{
		if (lpScrollData == 1)
		{
			SendDlgItemMessage( hDlg, IDC_SCROLL, BM_SETCHECK, BST_CHECKED, 0);
			g_bScroll = true;
		}
		else if (lpScrollData == 0)
		{
			g_bScroll = false;
			SendDlgItemMessage( hDlg, IDC_SCROLL, BM_SETCHECK, BST_UNCHECKED, 0);
		}
	}

	// Set logging default to unchecked
	SendDlgItemMessage( hDlg, IDC_LOG, BM_SETCHECK, BST_UNCHECKED, 0);
	SendDlgItemMessage( hDlg, IDC_LOGTO, WM_SETTEXT, 0, (LPARAM) TEXT("Not logging."));
	SendDlgItemMessage( hDlg, IDC_FILENAME, WM_SETTEXT, 0, (LPARAM) TEXT(""));
	StringCchCopy(g_LogFileName, 1, TEXT(""));

	if (g_hKey != NULL)
	{
		// If logging is enabled, set log to checked and set file name
		// if filename either does not exist on the system, or the value
		// for filename does not exist in the registry, default to not log
		if (RegQueryValueEx(g_hKey, TEXT("Log"),  NULL, &datatype, (BYTE*)&lpLogData, &lValueLog)==ERROR_SUCCESS)
		{
			if (lpLogData == 1)
			{
				if (RegQueryValueEx(g_hKey, TEXT("FileName"), NULL, &datatype2, (BYTE*)lpFileData, &lValueFile)==ERROR_SUCCESS)
				{
					if (lstrcmp(lpFileData, TEXT("")) != 0)
					{
						StringCchCopy(g_LogFileName, MAX_PATH, lpFileData );
						SendDlgItemMessage( hDlg, IDC_LOGTO, WM_SETTEXT, 0, (LPARAM) TEXT("Logging to:"));
						SendDlgItemMessage( hDlg, IDC_FILENAME, WM_SETTEXT, 0, (LPARAM) g_LogFileName);
						SendDlgItemMessage( hDlg, IDC_LOG, BM_SETCHECK, BST_CHECKED, 0);
						g_bLog = true;
					}
				}				
			}
		}
	}

	return S_OK;
}

HWND CreateListView (HWND hWndParent)
{
	HWND hWndListView;
	RECT rcl;
	INITCOMMONCONTROLSEX icex;

	// Ensure that the common control DLL is loaded.

	icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icex.dwICC  = ICC_LISTVIEW_CLASSES;
	InitCommonControlsEx(&icex);

	GetClientRect (hWndParent, &rcl);
	hWndListView = CreateWindow (WC_LISTVIEW, TEXT(""),
        ES_AUTOHSCROLL | WS_CHILD | WS_BORDER | LVS_REPORT | WS_VISIBLE,
		0, 0, 420, 150,
        hWndParent, HMENU(IDC_LIST1), NULL, NULL);
	SetupListBox( hWndListView );
	if (hWndListView == NULL)
		return NULL;
	return hWndListView;
}

//-----------------------------------------------------------------------------
// Name: SetupListBox()
// Desc:
//-----------------------------------------------------------------------------
VOID SetupListBox( HWND hwndList )
{	
    LV_COLUMN col;
    LONG iSubItem = 0;

	col.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    col.fmt = LVCFMT_LEFT;
    col.cx = 110;
    col.pszText = TEXT("DATE");
    col.cchTextMax = 100;
    col.iSubItem = iSubItem;
    ListView_InsertColumn(hwndList, iSubItem, &col);
    iSubItem++;

    col.pszText = TEXT("OP");
    col.cx = 28;
    col.iSubItem = iSubItem;
    ListView_InsertColumn(hwndList, iSubItem, &col);
    iSubItem++;

    col.pszText = TEXT("ARG");
    col.cx = 34;
    col.iSubItem = iSubItem;
    ListView_InsertColumn(hwndList, iSubItem, &col);
    iSubItem++;

    col.pszText = TEXT("ID");
    col.cx = 37;
    col.iSubItem = iSubItem;
    ListView_InsertColumn(hwndList, iSubItem, &col);
    iSubItem++;

    col.pszText = TEXT("DESCRIPTION");
    col.cx = 190;
    col.iSubItem = iSubItem;
    ListView_InsertColumn(hwndList, iSubItem, &col);
    iSubItem++;
}

//-----------------------------------------------------------------------------
// Name: CreateCaptureBuffer()
// Desc: Creates a capture buffer and sets the format
//-----------------------------------------------------------------------------
HRESULT CreateCaptureBuffer( WAVEFORMATEX* pwfxInput )
{
    HRESULT hr;
    DSCBUFFERDESC dscbd;

    SAFE_RELEASE( g_pDSNotify );
    SAFE_RELEASE( g_pDSBCapture );

    // Set the notification size
    g_dwNotifySize = 1024; 
    g_dwNotifySize -= g_dwNotifySize % pwfxInput->nBlockAlign;

    // Set the buffer sizes
    g_dwCaptureBufferSize = g_dwNotifySize * NUM_REC_NOTIFICATIONS;

    SAFE_RELEASE( g_pDSNotify );
    SAFE_RELEASE( g_pDSBCapture );

    // Create the capture buffer
    ZeroMemory( &dscbd, sizeof(dscbd) );
    dscbd.dwSize        = sizeof(dscbd);
    dscbd.dwBufferBytes = g_dwCaptureBufferSize;
    dscbd.lpwfxFormat   = pwfxInput; // Set the format during creatation

    if( FAILED( hr = g_pDSCapture->CreateCaptureBuffer( &dscbd,
                                                        &g_pDSBCapture,
                                                        NULL ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("CreateCaptureBuffer"), hr );

    g_dwNextCaptureOffset = 0;

    if( FAILED( hr = InitNotifications() ) )
        return DXTRACE_ERR_MSGBOX( TEXT("InitNotifications"), hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitNotifications()
// Desc: Inits the notifications on the capture buffer which are handled
//       in WinMain()
//-----------------------------------------------------------------------------
HRESULT InitNotifications()
{
    HRESULT hr;

    if( NULL == g_pDSBCapture )
        return E_FAIL;

    // Create a notification event, for when the sound stops playing
    if( FAILED( hr = g_pDSBCapture->QueryInterface( IID_IDirectSoundNotify,
                                                    (VOID**)&g_pDSNotify ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("QueryInterface"), hr );

    // Setup the notification positions
    for( INT i = 0; i < NUM_REC_NOTIFICATIONS; i++ )
    {
        g_aPosNotify[i].dwOffset = (g_dwNotifySize * i) + g_dwNotifySize - 1;
        g_aPosNotify[i].hEventNotify = g_hNotificationEvent;
    }

    // Tell DirectSound when to notify us. the notification will come in the from
    // of signaled events that are handled in WinMain()
    if( FAILED( hr = g_pDSNotify->SetNotificationPositions( NUM_REC_NOTIFICATIONS,
                                                            g_aPosNotify ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("SetNotificationPositions"), hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: StartOrStopRecord()
// Desc: Starts or stops the capture buffer from recording
//-----------------------------------------------------------------------------
HRESULT StartOrStopRecord( BOOL bStartRecording )
{
    HRESULT hr;

    if( bStartRecording )
    {
        // Create a capture buffer, and tell the capture
        // buffer to start recording
        if( FAILED( hr = CreateCaptureBuffer( &g_wfxInput ) ) )
            return DXTRACE_ERR_MSGBOX( TEXT("CreateCaptureBuffer"), hr );

        if( FAILED( hr = g_pDSBCapture->Start( DSCBSTART_LOOPING ) ) )
            return DXTRACE_ERR_MSGBOX( TEXT("Start"), hr );
    }
    else
    {
        // Stop the capture and read any data that
        // was not caught by a notification
        if( NULL == g_pDSBCapture )
            return S_OK;

        // Stop the buffer, and read any data that was not
        // caught by a notification
        if( FAILED( hr = g_pDSBCapture->Stop() ) )
            return DXTRACE_ERR_MSGBOX( TEXT("Stop"), hr );

        if( FAILED( hr = RecordCapturedData() ) )
            return DXTRACE_ERR_MSGBOX( TEXT("RecordCapturedData"), hr );
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RecordCapturedData()
// Desc: Copies data from the capture buffer to the output buffer
//-----------------------------------------------------------------------------
HRESULT RecordCapturedData()
{
    HRESULT hr;
    VOID*   pbCaptureData    = NULL;
    DWORD   dwCaptureLength;
    VOID*   pbCaptureData2   = NULL;
    DWORD   dwCaptureLength2;
    DWORD   dwReadPos;
    DWORD   dwCapturePos;
    LONG lLockSize;

    if( NULL == g_pDSBCapture )
        return S_FALSE;

    if( FAILED( hr = g_pDSBCapture->GetCurrentPosition( &dwCapturePos, &dwReadPos ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("GetCurrentPosition"), hr );

    lLockSize = dwReadPos - g_dwNextCaptureOffset;
    if( lLockSize < 0 )
        lLockSize += g_dwCaptureBufferSize;

    // Block align lock size so that we are always write on a boundary
    lLockSize -= (lLockSize % g_dwNotifySize);

    if( lLockSize == 0 )
        return S_FALSE;

    // Lock the capture buffer down
    if( FAILED( hr = g_pDSBCapture->Lock( g_dwNextCaptureOffset, lLockSize,
                                          &pbCaptureData, &dwCaptureLength,
                                          &pbCaptureData2, &dwCaptureLength2, 0L ) ) )
        return DXTRACE_ERR_MSGBOX( TEXT("Lock"), hr );

	// check for MDC
	result = mdc_decoder_process_samples(decoder, (unsigned char*)pbCaptureData, dwCaptureLength);

	if (result == 1)
		mdc_decoder_get_packet(decoder, &op, &arg, &unitID);

    // Move the capture offset along
    g_dwNextCaptureOffset += dwCaptureLength;
    g_dwNextCaptureOffset %= g_dwCaptureBufferSize; // Circular buffer

    if( pbCaptureData2 != NULL )
    {
		result2 = mdc_decoder_process_samples(decoder, (BYTE*)pbCaptureData2, sizeof((BYTE*)pbCaptureData2));
        // Move the capture offset along
        g_dwNextCaptureOffset += dwCaptureLength2;
        g_dwNextCaptureOffset %= g_dwCaptureBufferSize; // Circular buffer
    }

    // Unlock the capture buffer
    g_pDSBCapture->Unlock( pbCaptureData,  dwCaptureLength,
                           pbCaptureData2, dwCaptureLength2 );
    return S_OK;
}



